import { Injectable } from '@angular/core';
import { AlertController } from '@ionic/angular';
import * as Parse from 'parse';
import { AiChatService } from './aichart.service';
import { HttpService } from './http.service';
declare const AgoraRTC: any;

@Injectable({
  providedIn: 'root',
})
export class LiveService {
  options: {
    appid: string;
    channel: string;
    token: string;
  } = {
    appid: '71d3357d920d4352b39ec8b8d26a7cb9',
    channel: '',
    token: '',
  };
  localTracks: any = {
    audioTrack: null,
    videoTrack: null,
  };
  rid?: string; //房间id(channel)
  profile?: any = localStorage.getItem('profile');
  client: any; //客户端
  company: string = '';
  UID: any;
  tools: any = {
    audio: false, //是否关闭音频
    camera: false, //是否切换摄像头
    mute: false, //是否静音
  };
  user_published_list = new Set(); //已发布列表
  timer: any; //轮询获取频道token的定时
  alert: any; //提示框
  media_devices: {
    //音视频设备列表
    audioDevices: Array<any>;
    videoDevices: Array<any>;
  } = {
    audioDevices: [],
    videoDevices: [],
  };
  currentUsedDevice: { audioDevice: string; videoDevice: string } = {
    //当前使用的设备
    audioDevice: '',
    videoDevice: '',
  };
  room?: Parse.Object; //直播间
  constructor(
    private http: HttpService,
    private aiServ: AiChatService,
    private alertController: AlertController
  ) {
    this.client?.leave();
    this.company = this.aiServ.company;
    this.getProfile();
  }
  async getProfile() {
    if (this.profile) return;
    let queryProfile = new Parse.Query('Profile');
    queryProfile.equalTo('user', Parse.User.current()?.id);
    queryProfile.notEqualTo('isDeleted', true);
    queryProfile.equalTo('isCross', true);
    this.profile = await queryProfile.first();
    this.profile?.id &&
      localStorage.setItem('profile', JSON.stringify(this.profile.toJSON()));
  }
  /* 初始化Agora */
  initAgora() {
    this.timer && clearTimeout(this.timer);
    this.options['token'] = '';
    this.options['channel'] = '';
    this.client?.leave();
    this.client = AgoraRTC.createClient({ mode: 'rtc', codec: 'h264' });
    AgoraRTC.enableLogUpload();
  }
  // 获取所有音视频设备
  getDevices() {
    AgoraRTC.getDevices()
      .then((devices: any) => {
        this.media_devices.audioDevices = devices.filter(function (
          device: any
        ) {
          return device.kind === 'audioinput';
        });
        this.media_devices.videoDevices = devices.filter(function (
          device: any
        ) {
          return device.kind === 'videoinput';
        });
        console.log(this.media_devices);
        this.currentUsedDevice = {
          audioDevice: this.media_devices.audioDevices[0].deviceId,
          videoDevice: this.media_devices.videoDevices[0].deviceId,
        };
        return Promise.all([
          AgoraRTC.createCameraVideoTrack(),
          AgoraRTC.createMicrophoneAudioTrack(),
        ]);
      })
      .then((tracks: any) => {
        console.log(tracks);
      })
      .catch((err: any) => {
        console.warn('获取媒体权限失败', err);
        this.alertTips('获取媒体权限失败');
      });
  }

  async getToken(room: Parse.Object) {
    this.room = room;
    this.timer && clearTimeout(this.timer);
    let remoteEle = document.getElementById('vice-video');
    (remoteEle as any).style.display = 'none';
    //获取频道token记录
    if (room?.get('profile').id == this.profile) {
      this.UID = 111111;
      let uid = Parse.User.current()?.id;
      if (!uid) {
        this.timer = setTimeout(() => {
          this.getToken(room);
        }, 2000);
        return;
      }
      let baseurl = 'https://server.fmode.cn/api/webrtc/build_token';
      let reqBody = {
        company: this.company, // this.aiSer.company,
        profile: this.profile,
        channelName: this.profile,
      };
      let data: any = await this.http.httpRequst(baseurl, reqBody, 'POST');
      console.log(data);
      if (data.code == 200) {
        this.options.token = data.data.token;
        this.options.appid = data.data.appid;
        this.options.channel = room?.get('profile').id;
      }
    } else {
      let data = await this.updateToken(room?.get('profile').id);
      console.log(data);
      if (!data?.token) {
        this.timer = setTimeout(() => {
          this.getToken(room);
        }, 2000);
        return;
      }
      this.options.token = data.token;
      this.options.channel = data.channel;
    }
    this.join();
  }
  /* 获取token */
  async updateToken(pid: string) {
    let sql = `select  "rtc"."objectId" as "rid","rtc"."channel", "rtc"."token",  "rtc"."expiraTime" from "RtcToken" as "rtc" where "rtc"."profile" = '${pid}' and "rtc"."expiraTime" >= now() order by "createdAt" desc limit 1`;
    let tokenData: any = await this.http.customSQL(sql);
    if (
      tokenData &&
      tokenData.code == 200 &&
      tokenData.data &&
      tokenData.data.length > 0
    ) {
      return tokenData.data[0];
    } else {
      return null;
    }
  }
  // 进入频道
  async join() {
    let path = location.pathname;
    if (path.indexOf('/live/link-room/') == -1) return;
    console.log('频道：', this.options.channel);
    let data = await Promise.all([
      this.client.join(
        this.options.appid,
        this.options.channel,
        this.options.token,
        this.UID
      ),
      /* 创建音频和视频轨道 */
      AgoraRTC.createMicrophoneAudioTrack(),
      AgoraRTC.createCameraVideoTrack(),
    ]);
    // await this.client.setClientRole('host');
    this.localTracks.audioTrack = data[1];
    this.localTracks.videoTrack = data[2];
    // console.log(this.localTracks);
    let remoteEle = document.getElementById('vice-video');
    if (remoteEle) {
      remoteEle.textContent = '';
      remoteEle.style.display = 'block';
    }
    this.localTracks.videoTrack.play('vice-video'); //播放自己视频渲染
    if (this.tools['audio']) {
      this.localTracks.audioTrack.setEnabled(false);
    } else {
      this.localTracks.audioTrack.setEnabled(true);
    }
    await this.joinReady();
  }
  /* 发布本地视频 */
  async publishSelf() {
    await this.client.publish(Object.values(this.localTracks));
  }
  /* 订阅远程视频 */
  async joinReady() {
    await this.publishSelf();
    this.client.remoteUsers.forEach((user: any) => {
      console.log('remoteUsers', user.uid);
      this.client.subscribe(user, 'audio').then((audioTrack: any) => {
        this.user_published_list.add(user);
        audioTrack.setVolume(0);
        audioTrack.play();
      });
      this.client.subscribe(user, 'video').then((videoTrack: any) => {
        let remoteEle = document.getElementById('video');
        if (remoteEle) {
          remoteEle.textContent = '';
        }
        videoTrack.play('video');
      });
    });
    // this.timer && clearInterval(this.timer);
    // console.log(this.client.remoteUsers);
    // const isAnchor = this.room?.get('user')?.id === Parse.User.current()?.id
    // if(isAnchor){
    //   console.log('当前用户身份：房主');
    //   // this.localTracks.audioTrack.setEnabled(false);
    //   // this.localTracks.videoTrack.setEnabled(false);
    //   await this.publishSelf()
    // }else{
    //   console.log('当前用户身份:用户');
    //   console.log('主播在线状态',this.client.remoteUsers.length);
    //   if (location.pathname.indexOf('/live/link-room/') == -1) return;
    //   /* 如果主播还没有上线，等待主播上线后发布 */
    //   if(this.client.remoteUsers.length == 0){
    //     setTimeout(() => {
    //       this.joinReady()
    //     }, 1000);
    //     return
    //   }
    // }
    this.client.on('user-joined', (user: any) => {
      console.log(user, `${user.uid} 加入频道`);
    });
    this.client.on('user-published', async (user: any, mediaType: string) => {
      /* 当用户进入直播间，主播重新推流视频：解决订阅者remoteUsers无法渲染问题 */
      // if(isAnchor){
      //   // await this.client.unpublish(Object.values(this.localTracks));
      //   // await this.client.publish(Object.values(this.localTracks));
      //   // this.localTracks.videoTrack.setEnabled(true);
      //   // this.localTracks.audioTrack.setEnabled(true);
      // }
      console.log('用户推流成功', user);
      await this.client.subscribe(user, mediaType);
      let remoteEle = document.getElementById('video');
      if (remoteEle) {
        remoteEle.textContent = '';
      }
      if (mediaType === 'video') {
        user.videoTrack.play('video');
        this.alert?.dismiss();
      }
      if (mediaType === 'audio') {
        user.audioTrack.play();
        this.user_published_list.add(user);
      }
    });
    this.client.on('user-unpublished', async (user: any, mediaType: string) => {
      if (mediaType === 'audio') {
        console.log('对方已静音');
        this.user_published_list.delete(user);
      }
      if (mediaType === 'video') {
        console.log('用户取消推流');
        let remoteEle = document.getElementById('video');
        if (remoteEle) {
          remoteEle.textContent = '对方离开直播间';
        }
        this.alertTips('对方已离开直播间');
      }
    });
    // if(!isAnchor){
    //   this.publishSelf()
    // }
  }
  /* 停止音频推流 */
  async updatePublishedAudioTrack() {
    this.tools['audio'] = !this.tools['audio'];
    if (this.tools['audio'] && this.localTracks.audioTrack) {
      await this.localTracks.audioTrack.setEnabled(false);
      console.log('停止推送音频');
    } else {
      await this.localTracks.audioTrack.setEnabled(true);
      console.log('恢复推送音频');
    }
    // await this.client.unpublish(this.localTracks.audioTrack);
    return true;
  }
  /* 静音 */
  async muteAudio() {
    this.tools['mute'] = !this.tools['mute'];
    let list = Array.from(this.user_published_list);
    for (let index = 0; index < list.length; index++) {
      const user = list[index];
      if (this.tools['mute']) {
        console.log('静音');
        await this.client.unsubscribe(user, 'audio');
      } else {
        await this.client.subscribe(user, 'audio');
        console.log('恢复声音');
      }
    }
  }

  async alertTips(message: string, title?: string, callBack?: Function) {
    this.alert = await this.alertController.create({
      header: title || '提示',
      message: message,
      buttons: [
        {
          text: '确认',
          handler: () => {
            console.log('Confirm Cancel: blah');
            callBack && callBack();
          },
        },
      ],
    });
    await this.alert.present();
  }
}
